""":demand: F1.8"""

from typing import Optional, TypeVar
from .LinkedList import LinkedList

T = TypeVar('T')


class Stack:
    def __init__(self):
        self.__ll = LinkedList()

    @property
    def top(self) -> Optional['T']:
        """
        Returns the top of the stack.

        :return: The top value of the stack of any type
        """
        return self.__ll.head

    def push(self, value: T = None) -> None:
        """
        Pushes a value on the top of the stack.

        :param value: Any object
        :return: None
        """
        self.__ll.insert_head(value)

    def pop(self) -> Optional['T']:
        """
        Removes the top of the stack and returns it.

        :return: The top value of the stack of any type
        """
        if self.__ll.is_empty():
            raise IndexError("Stack is empty")
        value = self.__ll.head
        self.__ll.delete(0)
        return value

    def is_empty(self) -> bool:
        """
        Indicates whether the stack is empty.

        :return: A boolean
        """
        return self.__ll.is_empty()

    def __str__(self) -> str:
        return str(self.__ll)

    def __eq__(self, other) -> bool:
        return self.__ll == other.__ll

    def __to_visu__(self) -> str:
        return self.__ll.__to_visu__()


def Stack_top(stack: Stack) -> T:
    return stack.top


def Stack_is_empty(stack: Stack) -> bool:
    return stack.is_empty()


def Stack_push(stack: Stack, e: T) -> None:
    stack.push(e)


def Stack_pop(stack: Stack) -> T:
    return stack.pop()
